#!/usr/bin/env node

const fs = require('fs');
const beautifier = require('html-formatter');
const axios = require('axios');
const prompts = require('prompts');
const ydl = require('youtube-dl-exec')
const path = require('path');
const os = require('os')
const logger = require('progress-estimator')()

require('colors');

const beautify = async (data) => {
  const result = await beautifier.render(data);
  return result;
};

const sh = (cmd) => {
  const { execSync } = require('child_process');
  try { execSync(cmd, { stdio: 'inherit' }) }
  catch { /*console.log(e)*/ }
};

const vids=[];

const mkVidList = async (data) => {
  await JSON.parse(data).contents.twoColumnSearchResultsRenderer.primaryContents.sectionListRenderer.contents[0].itemSectionRenderer.contents.forEach(c => {
    if(c.videoRenderer) vids.push({
      title: c.videoRenderer.title.runs[0].text,
      url: 'https://www.youtube.com/watch?v=' + c.videoRenderer.videoId
    })
  });
  return vids;
}

const getData = async (searchTerm) => {
  const sanitizedSearchTerm = searchTerm.replace(/ /g, '+');
  const url = `https://www.youtube.com/results?search_query=${sanitizedSearchTerm}`;

  let data = await axios.get(url).then(res => { return res.data });
  data = await beautify(data);
  data = data.match(/ytInitialData\s=\s(.*)/);
  data = data.toString().replace('ytInitialData = ', '');
  data = data.replace(/;<\/script>.*/, '');

  const json = JSON.parse(JSON.stringify(data));
  const vidList = mkVidList(json);
  return vidList;
}

const getTitle = async (url) => {
  let data = await axios.get(url).then(res => { return res.data });
  data = await beautify(data);
  data = data.match(/ytInitialData\s=\s(.*)/);
  data = data.toString().replace('ytInitialData = ', '');
  data = JSON.parse(JSON.stringify(data.replace(/;<\/script>.*/, '')), null, 2);

  let vidTitle = JSON.parse(data).contents.twoColumnWatchNextResults.results.results.contents[0].videoPrimaryInfoRenderer.title.runs[0].text || false;
  return vidTitle || url.replace(/https?:\/\/(www.)?youtube.com\/watch\?v=/, '');
}



const spin = async (func) => {
  console.log();
  const result = await logger(func(), " Downloading video...")
  return result
}

const dV = async (url, path) => {
  const result = await spin(() => ydl(url, { output: path }))
  return result
}

const main = async (search) => {

  const response = search ? { searchTerm: search } : await prompts({
    type: 'text', name: 'searchTerm', message: 'Search: '
  });

  if(!response.searchTerm) process.exit(0);

  let help = false,
      to = false,
      auto = false,
      download = false,
      link = false;

  if(search) {

    if(response.searchTerm.match(/-h\s*?/) || response.searchTerm.match(/--help\s*?/)) if(response.searchTerm.split(/--?h(elp)?\s?/)[0]
      .replace(/--?to?\s/, '')
      .replace(/--?a(uto)?\s?/, '')
      .replace(/--?d(ownload)?\s?/, '')
      .replace(/-dt\s?/, '')
      .replace(/--?l(ink)?\s?/, '')
      .replace(/ /g, '') === '') {

      await sh(`echo -e "\n[33m  Usage: [36myt-cli [0;1m[options] [search-term]\n\n    [35m-a[0m | [35m--auto[0m     : auto select first result\n    [35m-d[0m | [35m--download[0m : download selected video to [3;1m\${HOME}/yt-cli[0m\n    [35m-t[0m | [35m--to[0m       : download selected video to specified directory\n    [35m-l[0m | [35m--link[0m     : Directly provide a URL/Link\n    [35m-h[0m | [35m--help[0m     : show this help message\n\n"`);

      help = true;
    } else {
      help = false;
    }

    if(response.searchTerm.match(/-t\s*/) || response.searchTerm.match(/--to\s*/)) if(response.searchTerm.split(/--?to?\s/)[0]
      .replace(/--?a(uto)?\s?/, '')
      .replace(/--?d(ownload)?\s?/, '')
      .replace(/-dt\s?/, '')
      .replace(/--?l(ink)?\s?/, '')
      .replace(/ /g, '') === '') {

      to = response.searchTerm.split(/--?to?\s/)[1] ? response.searchTerm.split(/--?to?\s/)[1].split(' ')[0] : false;
      response.searchTerm = response.searchTerm.replace(/--?to?\s/, '').replace(`${to}`, '').replace(/^\s+|\s+$/g, '');

    } else {
      to = false;
    }

    if(response.searchTerm.match(/-dt\s*/) || response.searchTerm.match(/--download-to\s*/)) if(response.searchTerm.split(/-dt\s/)[0]
      .replace(/--?a(uto)?\s?/, '')
      .replace(/--?d(ownload)?\s?/, '')
      .replace(/--?l(ink)?\s?/, '')
      .replace(/ /g, '') === '') {


      to = response.searchTerm.split(/-dt\s/)[1] ? response.searchTerm.split(/-dt\s/)[1].split(' ')[0] : false;
      response.searchTerm = response.searchTerm.replace(/-dt\s/, '').replace(`${to}`, '').replace(/^\s+|\s+$/g, '');

      download = true;

    } else {
      to = false;
    }

    if(response.searchTerm.match(/-a\s*?/) || response.searchTerm.match(/--auto\s*?/)) if(response.searchTerm.split(/--?a(uto)?\s?/)[0]
      .replace(/--?d(ownload)?\s?/, '')
      .replace(/-dt\s?/, '')
      .replace(/--?l(ink)?\s?/, '')
      .replace(/ /g, '') === '') {

      response.searchTerm = response.searchTerm.replace(/--?a(uto)?\s/, '');
      auto = true;
    }

    if(response.searchTerm.match(/-d\s*/) || response.searchTerm.match(/--download\s*?/)) if(response.searchTerm.split(/--?d(ownload)?\s?/)[0]
      .replace(/-dt\s?/, '')
      .replace(/--?l(ink)?\s?/, '')
      .replace(/ /g, '') === '') {

      response.searchTerm = response.searchTerm.replace(/--?d(ownload)?\s/, '');

      download = true;

    } else {
      download = false;
    }

    if(response.searchTerm.match(/-l\s*/) || response.searchTerm.match(/--link\s*?/)) if(response.searchTerm.split(/--?l(ink)?\s?/)[0]
      .replace(/ /g, '') === '') {

      response.url = response.searchTerm.replace(/--?l(ink)?\s/, '');
      response.searchTerm =  response.url;
      response.title = await getTitle(response.url);
      link = true;

    }


  } else {
    help = false;
    auto = false;
    download = false;
    to = false;
    link = false;
  }


  if(help) process.exit(0);

  if(to && !download) {
    await sh(`echo -e "\n[31m [-] [0;1mError: the [33m--to[0m flag cannot be used without the [33m--download[0m flag\n"`);
    process.exit(0);
  }

  let selectedVideo;

  if(link) {
    selectedVideo = {
      title: response.title,
      url: response.url
    }
  } else {

    const data = await getData(response.searchTerm);
    const list = data.map((d, i) => {
      return `${i + 1}. ${d.title}`;
    });

    const response2 = auto ? { video: 0 } : await prompts({
      type: 'select',
      name: 'video',
      message: 'Select video: ',
      choices: list,
      initial: 0
    });

    selectedVideo = data[response2.video];

  }

  if(download) {
    if(!to) to = os.homedir() + '/yt-cli';
    if(!fs.existsSync(to)) fs.mkdirSync(to);
    await dV(selectedVideo.url, to + '/' + selectedVideo.title + '.mp4');
    console.log(`\n[+] `.green + 'Video saved to \'' + (to + '/' + selectedVideo.title + '.mp4\'').cyan);
  } else sh(`mpv ${selectedVideo.url}`);

};

process.argv[2] ? main(process.argv.splice(2).join(' ')) : main();

